#include "srgp.h"
#include <stdio.h>

#define KEYMEASURE_SIZE			80
#define WINDOW_SIZE				400
#define NUM_STEPS				20

void DrawCurve(point *ControlPoints, int n)
{
	int i;
	float t, delta;
	point CurvePoints[n];

	CurvePoints[0].x = ControlPoints[0].x;	/* Bezier curves interpolate the first */
	CurvePoints[0].y = ControlPoints[0].y;	/* and last control points */
	delta = 1.0/n;					/* The curve is to be approximated by n points */
									/* t ranges from 0.0 to 1.0 */
	for( i = 1; i <= n; i++ ){
		t = i*delta;
		CurvePoints[i].x=ControlPoints[0].x*(1.0 - t)*(1.0 - t)*(1.0 - t)
			+ControlPoints[1].x*3.0*t*(1.0 - t)*(1.0 - t)
			+ControlPoints[2].x*3.0*t*t*(1.0 - t)
			+ControlPoints[3].x*t*t*t;

		CurvePoints[i].y=ControlPoints[0].y*(1.0 - t)*(1.0 - t)*(1.0 - t)
			+ControlPoints[1].y*3.0*t*(1.0 - t)*(1.0 - t)
			+ControlPoints[2].y*3.0*t*t*(1.0 - t)
			+ControlPoints[3].y*t*t*t;
	}
	SRGP_polyLine(n+1, CurvePoints);			/* Draw the complete curve */
}

main()
{
	locator_measure locMeasure,pastlocMeasure;
	char keyMeasure[KEYMEASURE_SIZE];
	int device;
	int numCtl;
	boolean terminate;
	rectangle screen;
	point ControlPoints[4];

	SRGP_begin("BezierCurves", WINDOW_SIZE, WINDOW_SIZE,1, FALSE);
	SRGP_setLocatorEchoType(CURSOR);
	SRGP_setLocatorButtonMask(LEFT_BUTTON_MASK | RIGHT_BUTTON_MASK);
	pastlocMeasure.position = SRGP_defPoint(-1, -1);	/* Initialize position to */
	SRGP_setLocatorMeasure(pastlocMeasure.position);	/* arbitrary location */
	SRGP_setKeyboardProcessingMode(RAW);
	SRGP_setInputMode(LOCATOR,EVENT);					/* Both locator (mouse) */
	SRGP_setInputMode(KEYBOARD,EVENT);					/* and keyboard are active */
	screen = SRGP_defRectangle(0, 0, WINDOW_SIZE-1, WINDOW_SIZE-1);

/* Main event loop */
	terminate = FALSE;
	do{
		device = SRGP_waitEvent(INDEFINITE);
		switch(device){
		case KEYBOARD:{
				SRGP_getKeyboard(keyMeasure, KEYMEASURE_SIZE);
				switch(keyMeasure[0]){
				case'q':								/* Quitting the program */
					terminate = TRUE;
					break;
				case'c':								/* Clearing the window */
					SRGP_setColor(0);
					SRGP_fillRectangle(screen);
					SRGP_setColor(1);
					break;
				}
				break;
			}												/*keyboard case */
		case LOCATOR:{
				SRGP_getLocator(&locMeasure);
				switch(locMeasure.buttonOfMostRecentTransition){
				case LEFT_BUTTON:			/* Defining remaining control points */
					if((locMeasure.buttonChord[LEFT_BUTTON] == DOWN) &&
						pastlocMeasure.position.x > 0){
						SRGP_setLocatorEchoRubberAnchor(locMeasure.position);
						SRGP_line(pastlocMeasure.position,locMeasure.position);
						pastlocMeasure = locMeasure;
						ControlPoints[numCtl] = locMeasure.position;
						numCtl++;
						if(numCtl == 4){
							SRGP_setLineStyle(CONTINUOUS);	/* To draw curve */
							SRGP_setLineWidth(2);
							DrawCurve(ControlPoints, NUM_STEPS);
							pastlocMeasure.position.x = -1;
							SRGP_setLocatorEchoType(CURSOR);
						}
						break;
				case RIGHT_BUTTON:			/* Start new set of control points */
						SRGP_setLocatorEchoRubberAnchor(locMeasure.position);
						pastlocMeasure = locMeasure;
						SRGP_setLocatorEchoType(RUBBER_LINE);
						SRGP_setLineStyle(DASHED);		/* To draw control polygon */
						SRGP_setLineWidth(1);
						ControlPoints[0] = locMeasure.position;
						numCtl = 1;
						break;
					}
				}				/* button handling */
			}					/* locator case */
		}						/* device switch */
	}while( !terminate );
	SRGP_end();
}