typedef float Coeffs[4][4][3];

void DrawSurface(Coeffs coefficients, int ns, int nt, int n)
	/* the variable coefficients are the coefficients for Q(s,t) */
	/* ns and nt are the number of curves of constant s and t to be drawn */
{
	float del, dels, delt, s, t;
	int i, j;
	/* Initialize */
	del = 1.0/n;		  /* Step size to use in drawing each curve */
	dels = 1.0/(ns - 1);  /* Step size in s when moving to next curve of constant t */
	delt = 1.0/(nt - 1);  /* Step size in t when moving to next curve of constant s */


	/* Draw ns curves of constant s, for s=0.0, dels, 2dels,... 1.0 */
	for( i = 0; i < ns; i++ ){
		s = i*dels;
		/* Draw a curve of constant s, varying t from 0.0 to 1.0 */
		/* X, Y, and Z are functions to evaluate the bicubics for a given s and t */
		MoveAbs3(X(s,0.0), Y(s,0.0), Z(s,0.0));
		for( j = 0; j < n ; j++){
			t = j*del;
			/* n steps are used as t varies from 0.0 to 1.0 for each curve */
			LineAbs3(X(s,t), Y(s,t), Z(s,t));
		}
	}

	/* Draw nt curves of constant t, for t=0.0, delt, 2delt,...1.0 */
	for( i = 0; i < nt; i++ ){
		t = i*delt;
		/* Draw a curve of constant t, varying s from 0.0 to 1.0 */
		MoveAbs3(X(0.0,t), Y(0.0,t), Z(0.0,t));
		for( j = 0; j < n; j++ ){
			s = j*del;
			/* n steps are used as s varies from 0 to 1 for each curve */
			LineAbs3(X(s,t), Y(s,t), Z(s,t));
		}
	}
}
